@@ -3,12 +3,8 @@ This module contains :class:`appyter.fields.Field`, the base class for all field
 defined in :mod:`appyter.profiles.default.fields`.
 ``` '''
 
-import os
-import re
-import json
-from copy import copy
 from flask import Markup
-from appyter.context import get_jinja2_env
+from appyter.ext.itertools import collapse
 
 class PartialField:
   ''' Partial instantiation of a field
@@ -21,7 +17,7 @@ class PartialField:
 
   def __call__(self, name=None, value=None, **kwargs):
     kwargs = dict(self._kwargs, **kwargs)
-    value = kwargs.pop('context').get(name)
+    value = kwargs.pop('context').get(name) if value is None else value
     return self._field(name=name, value=value, **kwargs)
 
 def build_fields(fields, context={}, env=None):
@@ -100,6 +96,22 @@ class Field(dict):
     '''
     return self['args']
 
+  def prepare(self, req):
+    ''' Given a flask request, capture relevant variables for this field
+    '''
+    data = {}
+    if type(req) == dict:
+      value = req.get(self.args['name']) if self.args['name'] in req else self.args.get('default')
+    elif req.json:
+      value = req.json.get(self.args['name']) if self.args['name'] in req.json else self.args.get('default')
+    elif req.form:
+      value = collapse(req.form.getlist(self.args['name'])) if self.args['name'] in req.form else self.args.get('default')
+    else:
+      raise NotImplementedError
+    #
+    self.args['value'] = value
+    return { self.args['name']: value }
+
   def constraint(self):
     ''' Return true if the received args.value satisfies constraints.
     Should be overridden by subclasses.
@@ -153,6 +165,7 @@ class Field(dict):
     ''' (SEMI-SAFE) Effective raw value of the field when parsed and constraints are asserted.
     When instantiating code, you should use safe_value.
     '''
+    print(self.args['name'], self.args['value'])
     choices = self.choices
     if self.raw_value is None:
       if not self.args.get('required'):
