@@ -22,35 +22,22 @@ async def app_runner(emitter, config):
   async def reload(changes):
     logger.info(f"Reload {changes}")
     async with state_lock:
-      proc = state.pop('proc')
-      proc.kill()
-      state['proc'] = await run_app(config)
+      if 'proc' in state:
+        proc = state.pop('proc')
+        proc.kill()
+        state['proc'] = await run_app(config)
   #
   @emitter.on('quit')
   async def quit():
     logger.info('Stopping app..')
-    if 'proc' in state:
+    async with state_lock:
       state.pop('proc').kill()
 
-async def try_n_times(n, coro, *args, **kwargs):
-  import asyncio
-  import traceback
-  backoff = 1
-  while n > 0:
-    try:
-      return await coro(*args, **kwargs)
-    except Exception as err:
-      n -= 1
-      if n == 0:
-        raise err
-      logger.warn(f"Failed to run start, trying again in {backoff}s...")
-      await asyncio.sleep(backoff)
-      backoff *= 2
-
 async def app_messager(emitter, config):
   import asyncio
-  from appyter.util import join_routes
+  from appyter.ext.urllib import join_slash
   from appyter.ext.socketio import AsyncClient
+  from appyter.ext.asyncio.try_n_times import try_n_times
   sio = AsyncClient()
   #
   @emitter.on('livereload')
@@ -64,7 +51,7 @@ async def app_messager(emitter, config):
     await sio.disconnect()
   #
   origin = f"http://{config['HOST']}:{config['PORT']}"
-  path = join_routes(config['PREFIX'], "socket.io")
+  path = join_slash(config['PREFIX'], "socket.io")
   logger.info(f"Connecting to appyter server at {origin}{path}...")
   await asyncio.sleep(1)
   await try_n_times(3, sio.connect, origin, socketio_path=path)
@@ -140,3 +127,4 @@ def serve(app_path, **kwargs):
   loop.run_forever()
   loop.run_until_complete(emitter.emit('quit'))
   loop.run_until_complete(emitter.flush())
+  loop.run_until_complete(emitter.clear())
