@@ -1,9 +1,8 @@
 import os
 import sys
 import click
+import fsspec
 import asyncio
-import nbformat as nbf
-import functools
 import datetime
 import traceback
 import logging
@@ -11,10 +10,11 @@ logger = logging.getLogger(__name__)
 
 from appyter import __version__
 from appyter.cli import cli
-from appyter.ext.fs import Filesystem
 from appyter.ext.nbclient import NotebookClientIOPubHook
 from appyter.parse.nb import nb_from_ipynb_io, nb_to_ipynb_io, nb_to_json
 from appyter.ext.click import click_option_setenv, click_argument_setenv
+from appyter.ext.urllib import join_url
+from appyter.ext.fsspec.fuse import fs_mount
 
 def cell_is_code(cell):
     return cell.get('cell_type') == 'code'
@@ -37,11 +37,10 @@ def json_emitter_factory(output):
   return json_emitter
 
 async def nbexecute_async(ipynb='', emit=json_emitter_factory(sys.stdout), cwd='', subscribe=None):
-  logger.info('nbexecute starting')
+  logger.info('starting')
   assert callable(emit), 'Emit must be callable'
-  with Filesystem(cwd, asynchronous=True) as fs:
-    with fs.open(ipynb, 'r') as fr:
-      nb = nb_from_ipynb_io(fr)
+  with fsspec.open(join_url(cwd, ipynb), 'r') as fr:
+    nb = nb_from_ipynb_io(fr)
   #
   if 'appyter' not in nb.metadata:
     logger.warn('detected legacy format, upgrading..')
@@ -75,13 +74,15 @@ async def nbexecute_async(ipynb='', emit=json_emitter_factory(sys.stdout), cwd='
   await emit({ 'type': 'status', 'data': 'Starting' })
   #
   try:
-    with Filesystem(cwd, with_path=True, asynchronous=True) as fs:
+    files = nb.metadata['appyter']['nbconstruct'].get('files')
+    logger.debug(f"{cwd=} {files=}")
+    async with fs_mount(cwd, pathmap=files, cached=True) as mnt:
       # setup execution_info with start time
       nb.metadata['appyter']['nbexecute']['started'] = datetime.datetime.now().replace(tzinfo=datetime.timezone.utc).isoformat()
-      with fs.open(ipynb, 'w') as fw:
+      with (mnt/ipynb).open('w') as fw:
         nb_to_ipynb_io(nb, fw)
       #
-      logger.info('nbexecute initializing')
+      logger.info('initializing')
       state = dict(progress=0, status='Starting')
       if callable(subscribe):
         await subscribe(lambda: dict(nb=nb_to_json(nb), **state))
@@ -93,7 +94,7 @@ async def nbexecute_async(ipynb='', emit=json_emitter_factory(sys.stdout), cwd='
           allow_errors=True,
           timeout=None,
           kernel_name='python3',
-          resources={ 'metadata': {'path': fs.path()} },
+          resources={ 'metadata': {'path': str(mnt) } },
           iopub_hook=iopub_hook,
         )
         await emit({ 'type': 'nb', 'data': nb_to_json(nb) })
@@ -103,14 +104,14 @@ async def nbexecute_async(ipynb='', emit=json_emitter_factory(sys.stdout), cwd='
             PATH=os.environ['PATH'],
           ),
         ):
-          logger.info('nbexecute executing')
+          logger.info('executing')
           state.update(status='Executing...', progress=0)
           await emit({ 'type': 'status', 'data': state['status'] })
           await emit({ 'type': 'progress', 'data': state['progress'] })
           n_cells = len(nb.cells)
           exec_count = 1
           for index, cell in enumerate(nb.cells):
-            logger.info(f"nbexecute executing cell {index}")
+            logger.debug(f"executing cell {index}")
             cell = await client.async_execute_cell(
               cell, index,
               execution_count=exec_count,
@@ -126,38 +127,50 @@ async def nbexecute_async(ipynb='', emit=json_emitter_factory(sys.stdout), cwd='
               state['status'] = 'Success'
               await emit({ 'type': 'status', 'data': state['status'] })
       except asyncio.CancelledError:
+        logger.info('cancelled')
         raise
       except Exception as e:
-        logger.info(f"nbexecute execution error")
+        logger.info('execution error')
         await emit({ 'type': 'error', 'data': str(e) })
       # Save execution completion time
-      logger.info('nbexecute saving')
+      logger.info('saving')
       nb.metadata['appyter']['nbexecute']['completed'] = datetime.datetime.now().replace(tzinfo=datetime.timezone.utc).isoformat()
       # save additional files
       # TODO: in the future we should individual files and include the original urls here
       nb.metadata['appyter']['nbexecute']['files'] = {
         path: path
-        for path in fs.ls()
-        if path != ipynb
+        for path in (
+          str(p.relative_to(mnt))
+          for p in mnt.rglob('*')
+          if p.is_file()
+        )
+        if path not in (files.keys()|{ipynb})
       }
       #
-      with fs.open(ipynb, 'w') as fw:
+      with (mnt/ipynb).open('w') as fw:
         nb_to_ipynb_io(nb, fw)
       #
+      logger.info('finalized')
   except asyncio.CancelledError:
+    logger.info(f"outer cancelled")
     raise
   except Exception as e:
-    await emit({ 'type': 'status', 'data': 'Error initializing, try again later' })
     logger.error(traceback.format_exc())
+    await emit({ 'type': 'status', 'data': 'Error initializing, try again later' })
   finally:
-    logger.info('nbexecute complete')
+    logger.info('complete')
   #
 
 @cli.command(help='Execute a jupyter notebook on the command line asynchronously')
 @click.option('-o', '--output', default='-', type=click.File('w'), help='The output location to write dynamic updates')
+@click_option_setenv('--data-dir', envvar='APPYTER_DATA_DIR', default=None, help='The directory that storage:// uris correspond to')
 @click_option_setenv('--cwd', envvar='APPYTER_CWD', default=os.getcwd(), help='The directory to treat as the current working directory for templates and execution')
 @click_argument_setenv('ipynb', envvar='APPYTER_IPYNB')
-def nbexecute(ipynb, output, cwd):
+def nbexecute(ipynb, output, cwd, data_dir=None):
+  if data_dir:
+    import fsspec
+    from appyter.ext.fsspec.alias import AliasFileSystemFactory
+    fsspec.register_implementation('storage', AliasFileSystemFactory('storage', data_dir))
   import asyncio
   loop = asyncio.get_event_loop()
   loop.run_until_complete(nbexecute_async(ipynb=ipynb, emit=json_emitter_factory(output), cwd=cwd))
